==> 0.1.3.tar.gz: test/model/field_list_test.dart <==
@TestGroup(description: 'FieldList')
library fields.test.model.field_list;

import 'dart:mirrors';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';
import 'package:fields/src/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

class HasFields {
  int i = 7;
  final int ri = 42;
  num n = 3.14;
  final num rn = 6.28;
  double d = 1024.4201;
  final double rd = 1.024;
  String s = 'snarf';
  final rs = 'immutable';
  List<int> li = [ 1, 2, 3, 4];
}

class HasMetadata {
  @UseFieldEditor(CustomIntEditor)    
  int customInt = 42;
  @ReadOnly
  double readOnlyDouble = 3.14;  
  @IgnoreField
  String secret = 'shhhh';
  @DisplayName('goodName')
  String badName = 'rename me please!';
}

class Proxy {
  final Object _o;
  final InstanceMirror _mirror;
  Proxy(Object o) : _o = o, _mirror = reflect(o);
  noSuchMethod(Invocation invocation) => _mirror.delegate(invocation);
  bool operator ==(Object other) => _o == other;
  int get hashCode => _o.hashCode;
  Type get runtimeType => _o.runtimeType;
  String toString() => _o.toString();
}

class TooManyFieldEditors {
  @UseFieldEditor(CustomIntEditor)
  @UseFieldEditor(CustomIntEditor)
  int customInt = 42;
}

class TooManyNames {
  @DisplayName('one')
  @DisplayName('too many')
  int customInt = 42;
}

class CustomIntEditor extends FieldEditor<int> {
  final element = null;
  CustomIntEditor(Field field) : super(field);
  void render() {}
}

// -----------------------------------------------------------------------------
// Basic
// -----------------------------------------------------------------------------

@Test('Test the construction of a field list.')
void testFieldListConstructor() {
  final list = new FieldList();
  expect(list.comparator, equals(FieldList.defaultComparator));
  expect(list.predicate, equals(FieldList.defaultPredicate));
  expect(list, isEmpty);
}

@Test('Test refreshing the source of a field list with default comparator'
      ' and predicate.')
void testRefreshSourceDefaults() {
  final obj = new HasFields();
  final list = new FieldList(); 
  list.refreshSource(obj);  
  expect(list.length, equals(9));  
  expect(list[0].displayName, equals('d'));
  expect(list[1].displayName, equals('i'));
  expect(list[2].displayName, equals('li'));
  expect(list[3].displayName, equals('n'));
  expect(list[4].displayName, equals('rd'));
  expect(list[5].displayName, equals('ri'));
  expect(list[6].displayName, equals('rn'));
  expect(list[7].displayName, equals('rs'));
  expect(list[8].displayName, equals('s'));
}

@Test('Test call to refreshSource with null argument.')
void testRefreshSourceNull() {
  final list = new FieldList(); 
  list.refreshSource(null);  
  // There are not really any assumptions we can make about the fields of Null:
  // https://code.google.com/p/dart/issues/detail?id=7502
  // So we are basically just verifying that it does not throw.
}

@Test('Test that setting a null predicate on a field list throws.')
void testSetNullPredicateThrows() {
  final list = new FieldList(); 
  expect(() {
    list.predicate = null;
  }, throwsA(const isInstanceOf<AssertionError>()));
}

@Test('Test that setting a null comparator on a field list throws.')
void testSetNullComparatorThrows() {
  final list = new FieldList(); 
  expect(() {
    list.comparator = null;
  }, throwsA(const isInstanceOf<AssertionError>()));
}

@Test('Test setting a reverse name comparator on a field list.')
void testSetReverseNameComparator() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);
  final Comparator reverse = 
      (Field a, Field b) => a.displayName.compareTo(b.displayName)*-1;    
  list.comparator = reverse;
  expect(list.comparator, equals(reverse));
  expect(list.length, equals(9));
  expect(list[0].displayName, equals('s'));
  expect(list[1].displayName, equals('rs'));
  expect(list[2].displayName, equals('rn'));
  expect(list[3].displayName, equals('ri'));
  expect(list[4].displayName, equals('rd'));
  expect(list[5].displayName, equals('n'));
  expect(list[6].displayName, equals('li'));
  expect(list[7].displayName, equals('i'));  
  expect(list[8].displayName, equals('d'));      
}

@Test('Test setting a predicate that accepts only read-only fields.')
void testSetReadOnlyPredicate() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);
  final Predicate readonly = (Field f) => f.isReadOnly;
  list.predicate = readonly;
  expect(list.predicate, equals(readonly));
  expect(list.length, equals(4));
  expect(list[0].displayName, equals('rd'));
  expect(list[1].displayName, equals('ri'));
  expect(list[2].displayName, equals('rn'));
  expect(list[3].displayName, equals('rs'));
}

@Test('Test the combination of setting a reverse name comparator and a'
      ' read-only predicate on a field list.')
void testSetReverseNameComparatorAndReadOnlyPredicate() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);
  final Comparator reverse = 
      (Field a, Field b) => a.displayName.compareTo(b.displayName)*-1;
  final Predicate readonly = (Field f) => f.isReadOnly;
  list.comparator = reverse;
  list.predicate = readonly;
  expect(list.length, equals(4));
  expect(list[0].displayName, equals('rs'));
  expect(list[1].displayName, equals('rn'));
  expect(list[2].displayName, equals('ri'));
  expect(list[3].displayName, equals('rd'));
}

@Test('Test setting the value of a field in the current view.')
void testSetFieldValueInView() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);  
  Field i = list[1];
  list.fieldChanged.listen(expectAsync1((f) {
    expect(f, equals(i));
  }));
  i.value = 42;
  expect(i.value, equals(42));
}

@Test('Test setting the value of a field not in the current view.')
void testSetFieldValueNotInView() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);
  Field i = list[1];
  final Predicate readonly = (Field f) => f.isReadOnly;
  list.predicate = readonly;
  expect(list, isNot(contains(i)));
  list.fieldChanged.listen((_) => fail('No field in view has changed.'));
  i.value = 42;
  expect(i.value, equals(42));
}

@Test('Test refreshing the source of a field list on a dirty object.')
void testRefreshSourceDirtyObject() {
  final obj = new HasFields();
  final list = new FieldList(); 
  list.refreshSource(obj);
  expect(list.length, equals(9));  
  list[0].values.listen(expectAsync1((value) {
    expect(value, equals(3.14));
  }));  
  // Modify a field of the object.
  obj.d = 3.14;  
  // Call to `refreshSource` on dirty object should trigger field value events.
  list.refreshSource(obj);
}

// -----------------------------------------------------------------------------
// Metadata
// -----------------------------------------------------------------------------

@Test('Test that successful parse of object field metadata.')
void testParseMetadata() {
  final obj = new HasMetadata();
  final list = new FieldList(); 
  list.refreshSource(obj);  
  expect(list.length, equals(3));  
  expect(list[0].name, equals(#customInt));
  // TODO(rms): Caught UnimplementedError: Mirror.hashCode is not implemented
  //expect(list[0].editor, equals(reflectClass(CustomIntEditor)));
  expect(list[1].name, equals(#badName));
  expect(list[1].displayName, equals('goodName'));  
  expect(list[2].name, equals(#readOnlyDouble));
  expect(list[2].isReadOnly, isTrue);  
}

@Test('Test that parsing multiple @UseFieldEditor annotations throws.')
void testParseMultipleUseFieldEditorAnnotationsThrows() {
  final obj = new TooManyFieldEditors();
  final list = new FieldList(); 
  expect(() {
    list.refreshSource(obj);
  }, throwsA(const isInstanceOf<AssertionError>()));  
}

@Test('Test that parsing multiple @Name annotations throws.')
void testParseMultipleNameAnnotationsThrows() {
  final obj = new TooManyNames();
  final list = new FieldList(); 
  expect(() {
    list.refreshSource(obj);
  }, throwsA(const isInstanceOf<AssertionError>()));  
}

// -----------------------------------------------------------------------------
// Proxy
// -----------------------------------------------------------------------------

@Test('Test refreshing the source of a field list with a proxy object.')
void testRefreshSourceProxy() {
  final obj = new HasFields();
  final proxy = new Proxy(obj);
  final list = new FieldList(); 
  list.refreshSource(proxy);
  expect(list.length, equals(9));  
  expect(list[0].displayName, equals('d'));
  expect(list[1].displayName, equals('i'));
  expect(list[2].displayName, equals('li'));
  expect(list[3].displayName, equals('n'));
  expect(list[4].displayName, equals('rd'));
  expect(list[5].displayName, equals('ri'));
  expect(list[6].displayName, equals('rn'));
  expect(list[7].displayName, equals('rs'));
  expect(list[8].displayName, equals('s'));
}

@Test('Test setting the value of a field via a proxy object.')
void testSetFieldValueProxy() {
  final obj = new HasFields();
  final proxy = new Proxy(obj);
  final list = new FieldList();
  list.refreshSource(proxy);  
  Field i = list[1];
  list.fieldChanged.listen(expectAsync1((f) {
    expect(f, equals(i));
  }));
  i.value = 42;
  expect(i.value, equals(42));
}

==> 0.4.0.tar.gz: test/model/field_list_test.dart <==
@TestGroup(description: 'FieldList')
library fields.test.model.field_list;

import 'dart:mirrors';
import 'package:bench/bench.dart';
import 'package:unittest/unittest.dart';
import 'package:fields/model.dart';

main() => reflectTests();

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------

class HasFields {
  int i = 7;
  final int ri = 42;
  num n = 3.14;
  final num rn = 6.28;
  double d = 1024.4201;
  final double rd = 1.024;
  String s = 'snarf';
  final rs = 'immutable';
  List<int> li = [ 1, 2, 3, 4];
}

class HasMetadata {
  @UseFieldEditor(CustomIntEditor)    
  int customInt = 42;
  @UseReadOnly
  double readOnlyDouble = 3.14;  
  @IgnoreField
  String secret = 'shhhh';
  @UseName('goodName')
  String badName = 'rename me please!';
}

class Proxy {
  final Object _o;
  final InstanceMirror _mirror;
  Proxy(Object o) : _o = o, _mirror = reflect(o);
  noSuchMethod(Invocation invocation) => _mirror.delegate(invocation);
  bool operator ==(Object other) => _o == other;
  int get hashCode => _o.hashCode;
  Type get runtimeType => _o.runtimeType;
  String toString() => _o.toString();
}

class TooManyFieldEditors {
  @UseFieldEditor(CustomIntEditor)
  @UseFieldEditor(CustomIntEditor)
  int customInt = 42;
}

class TooManyNames {
  @UseName('one')
  @UseName('too many')
  int customInt = 42;
}

class CustomIntEditor extends FieldEditor<int> {
  final element = null;
  CustomIntEditor(Field field) : super(field);
  void refreshElement() {}
}

// -----------------------------------------------------------------------------
// Basic
// -----------------------------------------------------------------------------

@Test('Test the construction of a field list.')
void testFieldListConstructor() {
  final list = new FieldList();
  expect(list.comparator, equals(FieldList.defaultComparator));
  expect(list.predicate, equals(FieldList.defaultPredicate));
  expect(list, isEmpty);
}

@Test('Test refreshing the source of a field list with default comparator'
      ' and predicate.')
void testRefreshSourceDefaults() {
  final obj = new HasFields();
  final list = new FieldList(); 
  list.refreshSource(obj);  
  expect(list.length, equals(9));  
  expect(list[0].name, equals('d'));
  expect(list[1].name, equals('i'));
  expect(list[2].name, equals('li'));
  expect(list[3].name, equals('n'));
  expect(list[4].name, equals('rd'));
  expect(list[5].name, equals('ri'));
  expect(list[6].name, equals('rn'));
  expect(list[7].name, equals('rs'));
  expect(list[8].name, equals('s'));
}

@Test('Test call to refreshSource with null argument.')
void testRefreshSourceNull() {
  final list = new FieldList(); 
  list.refreshSource(null);  
  // There are not really any assumptions we can make about the fields of Null:
  // https://code.google.com/p/dart/issues/detail?id=7502
  // So we are basically just verifying that it does not throw.
}

@Test('Test that setting a null predicate on a field list throws.')
void testSetNullPredicateThrows() {
  final list = new FieldList(); 
  expect(() {
    list.predicate = null;
  }, throwsA(const isInstanceOf<AssertionError>()));
}

@Test('Test that setting a null comparator on a field list throws.')
void testSetNullComparatorThrows() {
  final list = new FieldList(); 
  expect(() {
    list.comparator = null;
  }, throwsA(const isInstanceOf<AssertionError>()));
}

@Test('Test setting a reverse name comparator on a field list.')
void testSetReverseNameComparator() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);
  final Comparator reverse = 
      (Field a, Field b) => a.name.compareTo(b.name)*-1;    
  list.comparator = reverse;
  expect(list.comparator, equals(reverse));
  expect(list.length, equals(9));
  expect(list[0].name, equals('s'));
  expect(list[1].name, equals('rs'));
  expect(list[2].name, equals('rn'));
  expect(list[3].name, equals('ri'));
  expect(list[4].name, equals('rd'));
  expect(list[5].name, equals('n'));
  expect(list[6].name, equals('li'));
  expect(list[7].name, equals('i'));  
  expect(list[8].name, equals('d'));      
}

@Test('Test setting a predicate that accepts only read-only fields.')
void testSetReadOnlyPredicate() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);
  final Predicate readonly = (Field f) => f.isReadOnly;
  list.predicate = readonly;
  expect(list.predicate, equals(readonly));
  expect(list.length, equals(4));
  expect(list[0].name, equals('rd'));
  expect(list[1].name, equals('ri'));
  expect(list[2].name, equals('rn'));
  expect(list[3].name, equals('rs'));
}

@Test('Test the combination of setting a reverse name comparator and a'
      ' read-only predicate on a field list.')
void testSetReverseNameComparatorAndReadOnlyPredicate() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);
  final Comparator reverse = 
      (Field a, Field b) => a.name.compareTo(b.name)*-1;
  final Predicate readonly = (Field f) => f.isReadOnly;
  list.comparator = reverse;
  list.predicate = readonly;
  expect(list.length, equals(4));
  expect(list[0].name, equals('rs'));
  expect(list[1].name, equals('rn'));
  expect(list[2].name, equals('ri'));
  expect(list[3].name, equals('rd'));
}

@Test('Test setting the value of a field in the current view.')
void testSetFieldValueInView() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);  
  Field i = list[1];
  list.fieldChanged.listen(expectAsync1((f) {
    expect(f, equals(i));
  }));
  i.value = 42;
  expect(i.value, equals(42));
}

@Test('Test setting the value of a field not in the current view.')
void testSetFieldValueNotInView() {
  final obj = new HasFields();
  final list = new FieldList();
  list.refreshSource(obj);
  Field i = list[1];
  final Predicate readonly = (Field f) => f.isReadOnly;
  list.predicate = readonly;
  expect(list, isNot(contains(i)));
  list.fieldChanged.listen((_) => fail('No field in view has changed.'));
  i.value = 42;
  expect(i.value, equals(42));
}

@Test('Test refreshing the source of a field list on a dirty object.')
void testRefreshSourceDirtyObject() {
  final obj = new HasFields();
  final list = new FieldList(); 
  list.refreshSource(obj);
  expect(list.length, equals(9));  
  list[0].values.listen(expectAsync1((value) {
    expect(value, equals(3.14));
  }));  
  // Modify a field of the object.
  obj.d = 3.14;  
  // Call to `refreshSource` on dirty object should trigger field value events.
  list.refreshSource(obj);
}

// -----------------------------------------------------------------------------
// Metadata
// -----------------------------------------------------------------------------

@Test('Test that successful parse of object field metadata.')
void testParseMetadata() {
  final obj = new HasMetadata();
  final list = new FieldList(); 
  list.refreshSource(obj);  
  expect(list.length, equals(3));  
  expect(list[0].declaredName, equals(#customInt));
  // TODO(rms): Caught UnimplementedError: Mirror.hashCode is not implemented
  //expect(list[0].editor, equals(reflectClass(CustomIntEditor)));
  expect(list[1].declaredName, equals(#badName));
  expect(list[1].name, equals('goodName'));  
  expect(list[2].declaredName, equals(#readOnlyDouble));
  expect(list[2].isReadOnly, isTrue);  
}

@Test('Test that parsing multiple @UseFieldEditor annotations throws.')
void testParseMultipleUseFieldEditorAnnotationsThrows() {
  final obj = new TooManyFieldEditors();
  final list = new FieldList(); 
  expect(() {
    list.refreshSource(obj);
  }, throwsA(const isInstanceOf<AssertionError>()));  
}

@Test('Test that parsing multiple @Name annotations throws.')
void testParseMultipleNameAnnotationsThrows() {
  final obj = new TooManyNames();
  final list = new FieldList(); 
  expect(() {
    list.refreshSource(obj);
  }, throwsA(const isInstanceOf<AssertionError>()));  
}

// -----------------------------------------------------------------------------
// Proxy
// -----------------------------------------------------------------------------

@Test('Test refreshing the source of a field list with a proxy object.')
void testRefreshSourceProxy() {
  final obj = new HasFields();
  final proxy = new Proxy(obj);
  final list = new FieldList(); 
  list.refreshSource(proxy);
  expect(list.length, equals(9));  
  expect(list[0].name, equals('d'));
  expect(list[1].name, equals('i'));
  expect(list[2].name, equals('li'));
  expect(list[3].name, equals('n'));
  expect(list[4].name, equals('rd'));
  expect(list[5].name, equals('ri'));
  expect(list[6].name, equals('rn'));
  expect(list[7].name, equals('rs'));
  expect(list[8].name, equals('s'));
}

@Test('Test setting the value of a field via a proxy object.')
void testSetFieldValueProxy() {
  final obj = new HasFields();
  final proxy = new Proxy(obj);
  final list = new FieldList();
  list.refreshSource(proxy);  
  Field i = list[1];
  list.fieldChanged.listen(expectAsync1((f) {
    expect(f, equals(i));
  }));
  i.value = 42;
  expect(i.value, equals(42));
}
